//! PS/2 Keyboard Driver for AIOS v2

/// PS/2 keyboard data port
const KEYBOARD_DATA_PORT: u16 = 0x60;

/// PS/2 keyboard status port
const KEYBOARD_STATUS_PORT: u16 = 0x64;

/// Keyboard scan codes to ASCII mapping (US layout)
static SCANCODE_TO_ASCII: [u8; 128] = [
    0, 27, b'1', b'2', b'3', b'4', b'5', b'6', b'7', b'8', b'9', b'0', b'-', b'=', 8, // backspace
    b'\t', b'q', b'w', b'e', b'r', b't', b'y', b'u', b'i', b'o', b'p', b'[', b']', b'\n', // enter
    0, // left ctrl
    b'a', b's', b'd', b'f', b'g', b'h', b'j', b'k', b'l', b';', b'\'', b'`',
    0, // left shift
    b'\\', b'z', b'x', b'c', b'v', b'b', b'n', b'm', b',', b'.', b'/', 
    0, // right shift
    b'*',
    0, // left alt
    b' ', // space
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, // F1-F10
    0, // num lock
    0, // scroll lock
    0, 0, 0, 0, 0, 0, 0, 0, 0, // keypad
    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
];

/// Read a byte from a port
unsafe fn inb(port: u16) -> u8 {
    let result: u8;
    core::arch::asm!(
        "in al, dx",
        out("al") result,
        in("dx") port,
        options(nomem, nostack, preserves_flags)
    );
    result
}

/// Check if keyboard has data available
pub fn keyboard_ready() -> bool {
    unsafe {
        (inb(KEYBOARD_STATUS_PORT) & 0x01) != 0
    }
}

/// Read a key from the keyboard (non-blocking)
pub fn read_key() -> Option<u8> {
    if !keyboard_ready() {
        return None;
    }

    unsafe {
        let scancode = inb(KEYBOARD_DATA_PORT);
        
        // Only handle key press (bit 7 = 0), ignore key release
        if scancode & 0x80 == 0 {
            if let Some(&ascii) = SCANCODE_TO_ASCII.get(scancode as usize) {
                if ascii != 0 {
                    return Some(ascii);
                }
            }
        }
    }
    
    None
}