//! Safe Keyboard Driver for AIOS v2
//! Uses polling without interrupts to avoid conflicts

/// PS/2 keyboard data port
const KEYBOARD_DATA_PORT: u16 = 0x60;

/// PS/2 keyboard status port  
const KEYBOARD_STATUS_PORT: u16 = 0x64;

/// Simple scan code to ASCII mapping for common keys
fn scancode_to_ascii(scancode: u8) -> Option<u8> {
    match scancode {
        0x02 => Some(b'1'),
        0x03 => Some(b'2'),
        0x04 => Some(b'3'),
        0x05 => Some(b'4'),
        0x06 => Some(b'5'),
        0x07 => Some(b'6'),
        0x08 => Some(b'7'),
        0x09 => Some(b'8'),
        0x0A => Some(b'9'),
        0x0B => Some(b'0'),
        
        0x10 => Some(b'q'),
        0x11 => Some(b'w'),
        0x12 => Some(b'e'),
        0x13 => Some(b'r'),
        0x14 => Some(b't'),
        0x15 => Some(b'y'),
        0x16 => Some(b'u'),
        0x17 => Some(b'i'),
        0x18 => Some(b'o'),
        0x19 => Some(b'p'),
        
        0x1E => Some(b'a'),
        0x1F => Some(b's'),
        0x20 => Some(b'd'),
        0x21 => Some(b'f'),
        0x22 => Some(b'g'),
        0x23 => Some(b'h'),
        0x24 => Some(b'j'),
        0x25 => Some(b'k'),
        0x26 => Some(b'l'),
        
        0x2C => Some(b'z'),
        0x2D => Some(b'x'),
        0x2E => Some(b'c'),
        0x2F => Some(b'v'),
        0x30 => Some(b'b'),
        0x31 => Some(b'n'),
        0x32 => Some(b'm'),
        
        0x39 => Some(b' '), // space
        0x1C => Some(b'\n'), // enter
        0x0E => Some(8), // backspace
        
        _ => None,
    }
}

/// Safe port read with error handling
unsafe fn try_inb(port: u16) -> Option<u8> {
    // Add a simple delay to avoid rapid polling
    for _ in 0..1000 {
        core::arch::asm!("nop");
    }
    
    let result: u8;
    core::arch::asm!(
        "in al, dx",
        out("al") result,
        in("dx") port,
        options(nomem, nostack, preserves_flags)
    );
    Some(result)
}

/// Check if keyboard has data available (safe version)
pub fn keyboard_ready() -> bool {
    unsafe {
        if let Some(status) = try_inb(KEYBOARD_STATUS_PORT) {
            (status & 0x01) != 0
        } else {
            false
        }
    }
}

/// Read a key from keyboard with safety checks
pub fn read_key() -> Option<u8> {
    if !keyboard_ready() {
        return None;
    }
    
    unsafe {
        if let Some(scancode) = try_inb(KEYBOARD_DATA_PORT) {
            // Only handle key press (bit 7 = 0)
            if scancode & 0x80 == 0 {
                return scancode_to_ascii(scancode);
            }
        }
    }
    
    None
}

/// Get raw scancode for debugging
pub fn get_raw_scancode() -> Option<u8> {
    if !keyboard_ready() {
        return None;
    }
    
    unsafe {
        try_inb(KEYBOARD_DATA_PORT)
    }
}