//! Simple Shell for AIOS v2

use crate::{print_char, print_str, clear_screen};

/// Maximum command line length
const MAX_CMD_LEN: usize = 256;

/// Shell state
pub struct Shell {
    input_buffer: [u8; MAX_CMD_LEN],
    cursor: usize,
    prompt: &'static str,
}

impl Shell {
    /// Create a new shell instance
    pub fn new() -> Self {
        Shell {
            input_buffer: [0; MAX_CMD_LEN],
            cursor: 0,
            prompt: "aios> ",
        }
    }

    /// Print the shell prompt
    pub fn print_prompt(&self) {
        print_str(self.prompt, 0, get_current_row());
        set_cursor_col(self.prompt.len());
    }

    /// Process a single character input
    pub fn process_char(&mut self, c: u8) {
        match c {
            b'\n' => {
                // Enter pressed - execute command
                self.execute_command();
                self.clear_input();
                print_newline();
                self.print_prompt();
            }
            8 => {
                // Backspace
                if self.cursor > 0 {
                    self.cursor -= 1;
                    self.input_buffer[self.cursor] = 0;
                    self.redraw_line();
                }
            }
            32..=126 => {
                // Printable characters
                if self.cursor < MAX_CMD_LEN - 1 {
                    self.input_buffer[self.cursor] = c;
                    self.cursor += 1;
                    print_char(c, get_cursor_col(), get_current_row());
                    inc_cursor_col();
                }
            }
            _ => {
                // Ignore other characters
            }
        }
    }

    /// Execute the current command
    fn execute_command(&mut self) {
        let cmd = self.get_command_str();
        
        print_newline();
        
        match cmd {
            "help" => self.cmd_help(),
            "clear" => self.cmd_clear(),
            "echo" => self.cmd_echo(),
            "version" => self.cmd_version(),
            "reboot" => self.cmd_reboot(),
            "" => {
                // Empty command, do nothing
            }
            _ => {
                print_str("Unknown command: ", 0, get_current_row());
                print_str(cmd, 18, get_current_row());
                print_newline();
                print_str("Type 'help' for available commands.", 0, get_current_row());
                print_newline();
            }
        }
    }

    /// Get command as string slice
    fn get_command_str(&self) -> &str {
        let bytes = &self.input_buffer[..self.cursor];
        core::str::from_utf8(bytes).unwrap_or("")
    }

    /// Clear input buffer
    fn clear_input(&mut self) {
        self.input_buffer.fill(0);
        self.cursor = 0;
    }

    /// Redraw the current line
    fn redraw_line(&self) {
        let row = get_current_row();
        let prompt_len = self.prompt.len();
        
        // Clear the line after prompt
        for i in prompt_len..80 {
            print_char(b' ', i, row);
        }
        
        // Redraw the command
        for (i, &c) in self.input_buffer[..self.cursor].iter().enumerate() {
            print_char(c, prompt_len + i, row);
        }
        
        set_cursor_col(prompt_len + self.cursor);
    }

    // Command implementations
    fn cmd_help(&self) {
        print_str("AIOS v2 Shell Commands:", 0, get_current_row());
        print_newline();
        print_str("  help     - Show this help message", 0, get_current_row());
        print_newline();
        print_str("  clear    - Clear the screen", 0, get_current_row());
        print_newline();
        print_str("  echo     - Echo test message", 0, get_current_row());
        print_newline();
        print_str("  version  - Show AIOS version", 0, get_current_row());
        print_newline();
        print_str("  reboot   - Reboot the system", 0, get_current_row());
        print_newline();
    }

    fn cmd_clear(&self) {
        clear_screen();
        reset_cursor();
    }

    fn cmd_echo(&self) {
        print_str("Hello from AIOS v2 Shell!", 0, get_current_row());
        print_newline();
    }

    fn cmd_version(&self) {
        print_str("AIOS v2.0 - AI Operating System", 0, get_current_row());
        print_newline();
        print_str("Built with Rust and powered by Claude", 0, get_current_row());
        print_newline();
    }

    fn cmd_reboot(&self) {
        print_str("Rebooting system...", 0, get_current_row());
        print_newline();
        // Simple triple fault to reboot
        unsafe {
            core::arch::asm!("int3");
        }
    }
}

// VGA buffer interface (temporary - will be moved to separate module)
static mut CURSOR_ROW: usize = 0;
static mut CURSOR_COL: usize = 0;

fn get_current_row() -> usize {
    unsafe { CURSOR_ROW }
}

fn get_cursor_col() -> usize {
    unsafe { CURSOR_COL }
}

fn set_cursor_col(col: usize) {
    unsafe { CURSOR_COL = col; }
}

fn inc_cursor_col() {
    unsafe { 
        CURSOR_COL += 1;
        if CURSOR_COL >= 80 {
            CURSOR_COL = 0;
            CURSOR_ROW += 1;
            if CURSOR_ROW >= 25 {
                CURSOR_ROW = 24;
                // TODO: Scroll screen
            }
        }
    }
}

fn print_newline() {
    unsafe {
        CURSOR_ROW += 1;
        CURSOR_COL = 0;
        if CURSOR_ROW >= 25 {
            CURSOR_ROW = 24;
            // TODO: Scroll screen
        }
    }
}

fn reset_cursor() {
    unsafe {
        CURSOR_ROW = 0;
        CURSOR_COL = 0;
    }
}

